# -*- ruby -*-

#
# VFS for ruby - open anything with URI
# 
#   Copyright: Tatsuki Sugiura <sugi@nemui.org>
#   License: Ruby's original or GPL2
#   $LastChangedDate: 2003-10-08 17:56:18 +0900 (Wed, 08 Oct 2003) $
#   $LastChangedBy: sugi $
#

require 'net/http'

module Net
  class HTTP < Protocol
    def options( path, initheader = nil ) 
      res = request(Options.new(path, initheader))
      res.value unless @newimpl
      res
    end
    def propfind( path, initheader = { 'Depth' => '0' } )
      res = request(Propfind.new(path, initheader))
      res.value unless @newimpl
      res
    end
  end

  class HTTP
    class Options < HTTPRequest
      METHOD = 'OPTIONS'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = false
    end
    class Propfind < HTTPRequest
      METHOD = 'PROPFIND'
      REQUEST_HAS_BODY = false
      RESPONSE_HAS_BODY = true
    end
  end
end

module VFS;  module SchemeHandler;  module HTTP;
  module Common

    def check_uri_syntax(url)
      unless parse_host(url)
	raise Error::InvalidURI,
	  "Invalid HTTP URL '#{url}' was speficied."
      end
    end

    def init_common_attr
      @capability   = []
      @prop_cache   = nil
      @header       = nil
    end
    private :init_common_attr
    attr_writer :capability, :header
    attr_accessor :prop_cache
    private :prop_cache, :prop_cache=, :header=

    def capability?(method)
      capability.member? method      
    end

    def capability
      return @capability unless @capability.length == 0
      capability = http_fetch(url, :options) { |rep|
	rep["allow"].split(/\s*,\s*/)
      }
    end

    def property(elem)
      unless capability? "PROPFIND"
	return nil
      end
      if prop_cache == nil
	log("Start fetching property.")
	http_fetch(url, :propfind, {"Depth" => "0"}) { |rep|
	  log("*BUG/NotInplementedYet* must unescape XML entity.", Log::WARNING)
	  __send__ :prop_cache=, rep.body
	}
	log("Property was fetched")
      end
      m = Regexp.new("<#{elem}(?:\s+[^>]*)?>(.*?)</#{elem}>").match(prop_cache)
      if m && m[1]
	m[1].tr("\n\r\t", " ")
      else
	nil
      end
    end

    def header(name=nil)
      if @header == nil
	log("Start fetching header.")
	http_fetch(url, :head) { |rep|
	  __send__ :header=, rep.each_header{}
	}
	log("Header was fetched")
      end
      name == nil ? @header : @header[name]
    end

    def http_fetch(fetch_url, method=:get, *rest_args, &block)
      t_host = parse_host(fetch_url)
      t_port = parse_port(fetch_url)
      t_path = parse_path(fetch_url)
      t_auth = parse_auth(fetch_url) # not supported yet!
      
      Net::HTTP.version_1_1
      begin
	log("Opening HTTP connection.", Log::INFO)
	Net::HTTP::Proxy(parse_host(ENV["parse_proxy"]),
			 parse_port(ENV["parse_proxy"])).start(t_host,
							       t_port) { |http|
	  log("Requesting '#{method}' for '#{t_path}' to server.", Log::DEBUG)
	  response = http.send(method, t_path, *rest_args)
	  Array === response and response = response.first
	  log("Data was fetched from web server. Go yield.", Log::DEBUG)
	  yield(response)
	}
      rescue Net::ProtoRetriableError => err
	log("Retriable error was detected in HTTP connection. (detail: #{err})",
	    Log::INFO)
	l = err.response['location']
	if parse_host(l)
	  t_host = parse_host(l)
	  t_port = parse_port(l)
	  t_path = parse_path(l)
	  t_auth = parse_auth(l) # not supported yet!
	  # fxime; never returns when redirect loop.
	  retry
	end
      end
    end # http_fetch

    private
    def parse_path(url)
      Regexp.new("^http://[^/]+(.+)$").match(url).to_a[1]
    end
    
    def parse_host(url)
      Regexp.new("^http://(?:[^/@]+?@)?([^/]+)").match(url).to_a[1]
    end
    
    def parse_port(url)
      Regexp.new("^http://[^/]+:([0-9]+)/").match(url).to_a[1] || 80
    end
    
    def parse_auth(url)
      Regexp.new("^http://([^/]+?)@").match(url).to_a[1]
    end

  end # Common
end; end; end
